@@ -7,7 +7,7 @@ from lxml import etree
 from stream_unzip import UnzipValueError, stream_unzip
 
 
-def stream_read_ods(ods_chunks, max_string_length=65536, max_columns=65536, chunk_size=65536):
+def stream_read_ods(ods_chunks, max_string_length=65536, max_columns=65536, max_split_cells=65536, chunk_size=65536):
 
     # lxml iterparse takes a file-like object, but stream_read_ods accepts an iterable
     # so we have to do some low-ish level faffing to convert from one to the other
@@ -88,6 +88,19 @@ def stream_read_ods(ods_chunks, max_string_length=65536, max_columns=65536, chun
         def table_rows(parsed_xml_it):
             row = None
 
+            covered_cells = {}
+            i = 0
+            j = 0
+
+            def trim_trailing_nones(values):
+                # Excel ODS files output a _lot_ of trailing Nones
+                end = len(values)
+                for i in range(len(values) - 1, -1, -1):
+                    if values[i] is not None:
+                        break
+                    end = i
+                return tuple(values[0:end])
+
             while True:
                 event, element = _next(parsed_xml_it)
 
@@ -97,11 +110,61 @@ def stream_read_ods(ods_chunks, max_string_length=65536, max_columns=65536, chun
 
                 # Ending a row
                 if event == 'end' and f'{ns_table}table-row' == element.tag:
-                    yield tuple(row)
+                    yield trim_trailing_nones(row)
+                    i = 0
+                    j += 1
+
+                if event == 'start' and f'{ns_table}covered-table-cell' == element.tag:
+                    try:
+                        num_repeats = int(element.attrib.get(f'{ns_table}number-columns-repeated', '1'))
+                    except ValueError as e:
+                        raise InvalidODSXMLError from e
+
+                    for r in range(0, num_repeats):
+                        try:
+                            value = covered_cells.pop((i, j))
+                        except KeyError as e:
+                            raise InvalidODSXMLError from e
+                        _append(row, value)
+                        i += 1
 
                 # Starting a table cell
                 if event == 'start' and f'{ns_table}table-cell' == element.tag:
-                    _append(row, table_cell(parsed_xml_it, element))
+                    try:
+                        num_repeats = int(element.attrib.get(f'{ns_table}number-columns-repeated', '1'))
+                    except ValueError as e:
+                        raise InvalidODSXMLError from e
+
+                    try:
+                        num_col_spans = int(element.attrib.get(f'{ns_table}number-columns-spanned', '1'))
+                    except ValueError as e:
+                        raise InvalidODSXMLError from e
+
+                    try:
+                        num_row_spans = int(element.attrib.get(f'{ns_table}number-rows-spanned', '1'))
+                    except ValueError as e:
+                        raise InvalidODSXMLError from e
+
+                    if num_repeats > 1 and (num_col_spans > 1 or num_row_spans > 1):
+                        # Have not seen a real world example of this. For now, seems safer to fail
+                        raise InvalidODSXMLError('Cell row or column spanning combined with repeats is not supported')
+
+                    value = table_cell(parsed_xml_it, element)
+                    covered_cell_indexes = (
+                        (r, s)
+                        for r in range(0, num_col_spans)
+                        for s in range(0, num_row_spans)
+                        if r != 0 or s !=0
+                    )
+                    for r, s in covered_cell_indexes:
+                        if len(covered_cells) == max_split_cells:
+                            raise TooManySplitCells(max_split_cells)
+                        covered_cells[(i + r, j + s)] = value
+
+                    for r in range(0, num_repeats):
+                        _append(row, value)
+                        i += 1
+
 
                 # Ending the table
                 if event == 'end' and f'{ns_table}table' == element.tag:
@@ -174,7 +237,6 @@ def stream_read_ods(ods_chunks, max_string_length=65536, max_columns=65536, chun
                 stack = ['', '']
                 while True:
                     event, element = _next(parsed_xml_it)
-                    print(element)
 
                     if event == 'start' and previous_event == 'start':
                         stack.append('')
@@ -265,6 +327,7 @@ def stream_read_ods(ods_chunks, max_string_length=65536, max_columns=65536, chun
 
 
 def simple_table(rows, skip_rows=0):
+
     def up_to_first_none(values):
         vals = []
         for value in values:
@@ -383,5 +446,9 @@ class TooManyColumnsError(StreamReadODSError):
     pass
 
 
+class TooManySplitCells(StreamReadODSError):
+    pass
+
+
 class StringTooLongError(StreamReadODSError):
     pass
